==> videorbits-2.201.tgz: videorbits-2.201/src/simple_png_read.c <==

/*
 * this reads a png into a flat buffer.
 * 
 * if an alpha channel is not read in, one is created
 * and set to 255 (opaque).
 *
 * the buffer format is RGBA for 24bit color images,
 * and GA for 8bit greyscale images. these are the 
 * only 2 formats supported, and the alpha channel
 * is also 8bits.
 *
 * No reading comments for now, mostly since the
 * sophisticated keyword/value stuff is more than
 * orbits is prepared to use.
 *
 * returns 0 on success.
 *
 * NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE 
 ***********************************************
 * for simplicity, the caller MUST free() the  *
 * returned '*image' and '*comment' pointers!  *
 ***********************************************
 * 
 * maali@eyetap.org 
 */

#include <stdio.h>
#include <stdlib.h>
#include <png.h>

int simple_png_read(char *filename, 
    int *channels, unsigned char **image_ptr, 
    int *width, int *height, 
    int *moffset, int *noffset)
{
  FILE *fp;
  png_structp png_ptr;
  png_infop info_ptr;
  png_infop end_info;
  png_bytep *row_pointers;
  int number=8, is_png;
  int bitdepth, colortype;
  char header[9];
  int retval=0;
  int i;


  if (!(fp=fopen(filename, "rb"))) { 
    fprintf(stderr, "simple_png_read: "
        "Can't open file '%s' for reading.\n", 
        filename); 
    return -1; 
  }

  fread(header, 1, number, fp);
  is_png = !png_sig_cmp(header, 0, number);
  if (!is_png) {
    fprintf(stderr, "simple_png_read: "
        "Not a PNG file ('%s').\n",
        filename); 
    return -1;
  }

  png_ptr = png_create_read_struct (
        PNG_LIBPNG_VER_STRING, 
        NULL, //(png_voidp)user_error_ptr,
        NULL, //user_error_fn, 
        NULL);//user_warning_fn);
  if (!png_ptr) {
    fprintf(stderr, "simple_png_read: error (no mem?A)\n"); 
    return -1;
  }
  
  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    fprintf(stderr, "simple_png_read: error (no mem?B)\n"); 
    png_destroy_read_struct(
        &png_ptr,
        (png_infopp)NULL, 
        (png_infopp)NULL);
    return -1;
  }
  
  end_info = png_create_info_struct(png_ptr);
  if (!end_info) {
    fprintf(stderr, "simple_png_read: error (no mem?C)\n"); 
    png_destroy_read_struct(
        &png_ptr, 
        &info_ptr,
        (png_infopp)NULL);
    return -1;
  }

  if (setjmp(png_ptr->jmpbuf)) {
    fprintf(stderr, "simple_png_read: error (internal)\n"); 
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
    fclose(fp);
    return -1;
  }

  // select file, inform lib 'number' chars have been read
  png_init_io(png_ptr, fp);
  png_set_sig_bytes(png_ptr, number);

  // read in all header data
  png_read_info(png_ptr, info_ptr); 

  // use 'png_get_IHDR' to read header values, last 3 
  // params: interlace_type, compression_type, filter_type
  png_get_IHDR(png_ptr,info_ptr,(png_uint_32*)width,(png_uint_32*)height,
    &bitdepth, &colortype, NULL, NULL, NULL);
  *moffset=png_get_x_offset_pixels(png_ptr,info_ptr);
  *noffset=png_get_y_offset_pixels(png_ptr,info_ptr);

  // check bitdepth/colortype is grey=8 or rgb=24, with or without alpha.
  if (!(
        ((colortype==PNG_COLOR_TYPE_GRAY)       &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_GRAY_ALPHA) &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_RGB)        &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_RGB_ALPHA)  &&(bitdepth==8))
     ))
  {
    fprintf(stderr, "simple_png_read: "
        "Unsupported PNG file type ('%s'). \n"
        "simple_png_read: "
        "File must be 8 bits per channel, greyscale \n"
        "simple_png_read: "
        "or rgb, with or without alpha.\n",
        filename); 
    return -1;
  }

  // if without alpha, use a filler byte 255 (after... rgba, ga)
  *channels = png_get_channels(png_ptr,info_ptr);
  if (  (colortype==PNG_COLOR_TYPE_GRAY)
      ||(colortype==PNG_COLOR_TYPE_RGB)) {
    png_set_filler(png_ptr, 255, PNG_FILLER_AFTER); 
    (*channels)++;
  }

  // use 'png_get_text' to get comments
  //numtext = png_get_text(png_ptr, info_ptr, &text_ptr);
  //text_ptr           - array of png_text holding image comments
  //text_ptr[i]->key   - keyword for comment.
  //text_ptr[i]->text  - text comments for current keyword.

  if (image_ptr) {
    // allocate image buffer and row_pointers to point into the buffer
    *image_ptr=NULL;
    retval = (*width)*(*height)*(*channels);
    *image_ptr = (unsigned char*)malloc((*width)*(*height)*(*channels));
    row_pointers = (png_bytep*)malloc((*height)*sizeof(png_bytep));
    for (i=1, row_pointers[0]=(*image_ptr); i<(*height); i++) { 
      row_pointers[i] = row_pointers[i-1]+((*width)*(*channels));
    }
  
    png_read_image(png_ptr, row_pointers);
    png_read_end(png_ptr, end_info);
    free(row_pointers);
  }
  // reload channels
  *channels = png_get_channels(png_ptr,info_ptr);

  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
  
  if (  (colortype==PNG_COLOR_TYPE_GRAY_ALPHA)
      ||(colortype==PNG_COLOR_TYPE_RGB_ALPHA)) {
    (*channels)--; // don't report alpha channel 
  }

  fclose(fp);
  //return retval;
  return 0;
}


int simple_png_read_header(FILE *fp,
    int *channels, 
    int *width, int *height, 
    int *moffset, int *noffset)
{
  png_structp png_ptr;
  png_infop info_ptr;
  png_infop end_info;
  int number=8, is_png;
  int bitdepth, colortype;
  char header[9];

  fread(header, 1, number, fp);
  is_png = !png_sig_cmp(header, 0, number);
  if (!is_png) {
    fprintf(stderr, "simple_png_read_header: "
        "Not a PNG file ('%s').\n",
        ""); 
    return -1;
  }

  png_ptr = png_create_read_struct (
        PNG_LIBPNG_VER_STRING, 
        NULL, //(png_voidp)user_error_ptr,
        NULL, //user_error_fn, 
        NULL);//user_warning_fn);
  if (!png_ptr) {
    fprintf(stderr, "simple_png_read_header: error (no mem?A)\n"); 
    return -1;
  }
  
  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    fprintf(stderr, "simple_png_read_header: error (no mem?B)\n"); 
    png_destroy_read_struct(
        &png_ptr,
        (png_infopp)NULL, 
        (png_infopp)NULL);
    return -1;
  }
  
  end_info = png_create_info_struct(png_ptr);
  if (!end_info) {
    fprintf(stderr, "simple_png_read_header: error (no mem?C)\n"); 
    png_destroy_read_struct(
        &png_ptr, 
        &info_ptr,
        (png_infopp)NULL);
    return -1;
  }

  if (setjmp(png_ptr->jmpbuf)) {
    fprintf(stderr, "simple_png_read_header: error\n"); 
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
    fclose(fp);
    return -1;
  }

  // select file, inform lib 'number' chars have been read
  png_init_io(png_ptr, fp);
  png_set_sig_bytes(png_ptr, number);

  // read in all header data
  png_read_info(png_ptr, info_ptr); 

  // use 'png_get_IHDR' to read header values, last 3 
  // params: interlace_type, compression_type, filter_type
  png_get_IHDR(png_ptr,info_ptr,(png_uint_32*)width,(png_uint_32*)height,
    &bitdepth, &colortype, NULL, NULL, NULL);
  *moffset=png_get_x_offset_pixels(png_ptr,info_ptr);
  *noffset=png_get_y_offset_pixels(png_ptr,info_ptr);

  // check bitdepth/colortype is grey=8 or rgb=24, with or without alpha.
  if (!(
        ((colortype==PNG_COLOR_TYPE_GRAY)       &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_GRAY_ALPHA) &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_RGB)        &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_RGB_ALPHA)  &&(bitdepth==8))
     ))
  {
    fprintf(stderr, "simple_png_read_header: "
        "Unsupported PNG file type ('%s'). \n"
        "simple_png_read_header: "
        "File must be 8 bits per channel, greyscale \n"
        "simple_png_read_header: "
        "or rgb, with or without alpha.\n",
        ""); 
    return -1;
  }

  // if without alpha, use a filler byte 255 (after... rgba, ga)
  if (  (colortype==PNG_COLOR_TYPE_GRAY)
      ||(colortype==PNG_COLOR_TYPE_RGB)) {
    png_set_filler(png_ptr, 255, PNG_FILLER_AFTER); 
  }
  *channels = png_get_channels(png_ptr,info_ptr);

  // use 'png_get_text' to get comments
  //numtext = png_get_text(png_ptr, info_ptr, &text_ptr);
  //text_ptr           - array of png_text holding image comments
  //text_ptr[i]->key   - keyword for comment.
  //text_ptr[i]->text  - text comments for current keyword.


  //png_read_end(png_ptr, end_info);
  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
  
  if (  (colortype==PNG_COLOR_TYPE_GRAY_ALPHA)
      ||(colortype==PNG_COLOR_TYPE_RGB_ALPHA)) {
    (*channels)--; // don't report alpha channel 
  }
  return 0;
}

==> videorbits-2.205.tgz: videorbits-2.205/src/simple_png_read.c <==

/*
 * this reads a png into a flat buffer.
 * 
 * if an alpha channel is not read in, one is created
 * and set to 255 (opaque).
 *
 * the buffer format is RGBA for 24bit color images,
 * and GA for 8bit greyscale images. these are the 
 * only 2 formats supported, and the alpha channel
 * is also 8bits.
 *
 * No reading comments for now, mostly since the
 * sophisticated keyword/value stuff is more than
 * orbits is prepared to use.
 *
 * returns 0 on success.
 *
 * NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE 
 ***********************************************
 * for simplicity, the caller MUST free() the  *
 * returned '*image' and '*comment' pointers!  *
 ***********************************************
 * 
 * maali@eyetap.org 
 */

#include <stdio.h>
#include <stdlib.h>
#include <png.h>

int simple_png_read(char *filename, 
    int *channels, unsigned char **image_ptr, 
    int *width, int *height, 
    int *moffset, int *noffset)
{
  FILE *fp;
  png_structp png_ptr;
  png_infop info_ptr;
  png_infop end_info;
  png_bytep *row_pointers;
  int number=8, is_png;
  int bitdepth, colortype;
  char header[9];
  int retval=0;
  int i;


  if (!(fp=fopen(filename, "rb"))) { 
    fprintf(stderr, "simple_png_read: "
        "Can't open file '%s' for reading.\n", 
        filename); 
    return -1; 
  }

  fread(header, 1, number, fp);
  is_png = !png_sig_cmp((png_byte *)header, 0, number);
  if (!is_png) {
    fprintf(stderr, "simple_png_read: "
        "Not a PNG file ('%s').\n",
        filename); 
    return -1;
  }

  png_ptr = png_create_read_struct (
        PNG_LIBPNG_VER_STRING, 
        NULL, //(png_voidp)user_error_ptr,
        NULL, //user_error_fn, 
        NULL);//user_warning_fn);
  if (!png_ptr) {
    fprintf(stderr, "simple_png_read: error (no mem?A)\n"); 
    return -1;
  }
  
  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    fprintf(stderr, "simple_png_read: error (no mem?B)\n"); 
    png_destroy_read_struct(
        &png_ptr,
        (png_infopp)NULL, 
        (png_infopp)NULL);
    return -1;
  }
  
  end_info = png_create_info_struct(png_ptr);
  if (!end_info) {
    fprintf(stderr, "simple_png_read: error (no mem?C)\n"); 
    png_destroy_read_struct(
        &png_ptr, 
        &info_ptr,
        (png_infopp)NULL);
    return -1;
  }

  if (setjmp(png_ptr->jmpbuf)) {
    fprintf(stderr, "simple_png_read: error (internal)\n"); 
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
    fclose(fp);
    return -1;
  }

  // select file, inform lib 'number' chars have been read
  png_init_io(png_ptr, fp);
  png_set_sig_bytes(png_ptr, number);

  // read in all header data
  png_read_info(png_ptr, info_ptr); 

  // use 'png_get_IHDR' to read header values, last 3 
  // params: interlace_type, compression_type, filter_type
  png_get_IHDR(png_ptr,info_ptr,(png_uint_32*)width,(png_uint_32*)height,
    &bitdepth, &colortype, NULL, NULL, NULL);
  *moffset=png_get_x_offset_pixels(png_ptr,info_ptr);
  *noffset=png_get_y_offset_pixels(png_ptr,info_ptr);

  // check bitdepth/colortype is grey=8 or rgb=24, with or without alpha.
  if (!(
        ((colortype==PNG_COLOR_TYPE_GRAY)       &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_GRAY_ALPHA) &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_RGB)        &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_RGB_ALPHA)  &&(bitdepth==8))
     ))
  {
    fprintf(stderr, "simple_png_read: "
        "Unsupported PNG file type ('%s'). \n"
        "simple_png_read: "
        "File must be 8 bits per channel, greyscale \n"
        "simple_png_read: "
        "or rgb, with or without alpha.\n",
        filename); 
    return -1;
  }

  // if without alpha, use a filler byte 255 (after... rgba, ga)
  *channels = png_get_channels(png_ptr,info_ptr);
  if (  (colortype==PNG_COLOR_TYPE_GRAY)
      ||(colortype==PNG_COLOR_TYPE_RGB)) {
    png_set_filler(png_ptr, 255, PNG_FILLER_AFTER); 
    (*channels)++;
  }

  // use 'png_get_text' to get comments
  //numtext = png_get_text(png_ptr, info_ptr, &text_ptr);
  //text_ptr           - array of png_text holding image comments
  //text_ptr[i]->key   - keyword for comment.
  //text_ptr[i]->text  - text comments for current keyword.

  if (image_ptr) {
    // allocate image buffer and row_pointers to point into the buffer
    *image_ptr=NULL;
    retval = (*width)*(*height)*(*channels);
    *image_ptr = (unsigned char*)malloc((*width)*(*height)*(*channels));
    row_pointers = (png_bytep*)malloc((*height)*sizeof(png_bytep));
    for (i=1, row_pointers[0]=(*image_ptr); i<(*height); i++) { 
      row_pointers[i] = row_pointers[i-1]+((*width)*(*channels));
    }
  
    png_read_image(png_ptr, row_pointers);
    png_read_end(png_ptr, end_info);
    free(row_pointers);
  }
  // reload channels
  *channels = png_get_channels(png_ptr,info_ptr);

  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
  
  if (  (colortype==PNG_COLOR_TYPE_GRAY_ALPHA)
      ||(colortype==PNG_COLOR_TYPE_RGB_ALPHA)) {
    (*channels)--; // don't report alpha channel 
  }

  fclose(fp);
  //return retval;
  return 0;
}


int simple_png_read_header(FILE *fp,
    int *channels, 
    int *width, int *height, 
    int *moffset, int *noffset)
{
  png_structp png_ptr;
  png_infop info_ptr;
  png_infop end_info;
  int number=8, is_png;
  int bitdepth, colortype;
  char header[9];

  fread(header, 1, number, fp);
  is_png = !png_sig_cmp((png_byte *)header, 0, number);
  if (!is_png) {
    fprintf(stderr, "simple_png_read_header: "
        "Not a PNG file ('%s').\n",
        ""); 
    return -1;
  }

  png_ptr = png_create_read_struct (
        PNG_LIBPNG_VER_STRING, 
        NULL, //(png_voidp)user_error_ptr,
        NULL, //user_error_fn, 
        NULL);//user_warning_fn);
  if (!png_ptr) {
    fprintf(stderr, "simple_png_read_header: error (no mem?A)\n"); 
    return -1;
  }
  
  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    fprintf(stderr, "simple_png_read_header: error (no mem?B)\n"); 
    png_destroy_read_struct(
        &png_ptr,
        (png_infopp)NULL, 
        (png_infopp)NULL);
    return -1;
  }
  
  end_info = png_create_info_struct(png_ptr);
  if (!end_info) {
    fprintf(stderr, "simple_png_read_header: error (no mem?C)\n"); 
    png_destroy_read_struct(
        &png_ptr, 
        &info_ptr,
        (png_infopp)NULL);
    return -1;
  }

  if (setjmp(png_ptr->jmpbuf)) {
    fprintf(stderr, "simple_png_read_header: error\n"); 
    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
    fclose(fp);
    return -1;
  }

  // select file, inform lib 'number' chars have been read
  png_init_io(png_ptr, fp);
  png_set_sig_bytes(png_ptr, number);

  // read in all header data
  png_read_info(png_ptr, info_ptr); 

  // use 'png_get_IHDR' to read header values, last 3 
  // params: interlace_type, compression_type, filter_type
  png_get_IHDR(png_ptr,info_ptr,(png_uint_32*)width,(png_uint_32*)height,
    &bitdepth, &colortype, NULL, NULL, NULL);
  *moffset=png_get_x_offset_pixels(png_ptr,info_ptr);
  *noffset=png_get_y_offset_pixels(png_ptr,info_ptr);

  // check bitdepth/colortype is grey=8 or rgb=24, with or without alpha.
  if (!(
        ((colortype==PNG_COLOR_TYPE_GRAY)       &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_GRAY_ALPHA) &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_RGB)        &&(bitdepth==8))
      ||((colortype==PNG_COLOR_TYPE_RGB_ALPHA)  &&(bitdepth==8))
     ))
  {
    fprintf(stderr, "simple_png_read_header: "
        "Unsupported PNG file type ('%s'). \n"
        "simple_png_read_header: "
        "File must be 8 bits per channel, greyscale \n"
        "simple_png_read_header: "
        "or rgb, with or without alpha.\n",
        ""); 
    return -1;
  }

  // if without alpha, use a filler byte 255 (after... rgba, ga)
  if (  (colortype==PNG_COLOR_TYPE_GRAY)
      ||(colortype==PNG_COLOR_TYPE_RGB)) {
    png_set_filler(png_ptr, 255, PNG_FILLER_AFTER); 
  }
  *channels = png_get_channels(png_ptr,info_ptr);

  // use 'png_get_text' to get comments
  //numtext = png_get_text(png_ptr, info_ptr, &text_ptr);
  //text_ptr           - array of png_text holding image comments
  //text_ptr[i]->key   - keyword for comment.
  //text_ptr[i]->text  - text comments for current keyword.


  //png_read_end(png_ptr, end_info);
  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
  
  if (  (colortype==PNG_COLOR_TYPE_GRAY_ALPHA)
      ||(colortype==PNG_COLOR_TYPE_RGB_ALPHA)) {
    (*channels)--; // don't report alpha channel 
  }
  return 0;
}
